@@ -566,9 +566,13 @@ def go_(
     job_id = res["job_id"]
     aws_config: Optional[AWSConfig] = res.get("aws_config", None)
 
-    sc = read_shell_config()
-    if sc.docker_password and sc.docker_username:
-        do_docker_login(sc.docker_username, sc.docker_password)
+    docker_username = os.environ.get("DOCKER_USERNAME", None)
+    docker_password = os.environ.get("DOCKER_PASSWORD", None)
+
+    if docker_password and docker_username:
+        do_docker_login(docker_username, docker_password)
+    else:
+        logger.warning("Docker credentials not found in environment")
     #
     # handler = TermHandler(
     #     token=token,
@@ -1076,186 +1080,203 @@ def run_one(
         run_docker(wd, project, cmd)
         logger.debug(f"Creating containers {services}")
         cmd = ["up", "--no-start", "--renew-anon-volumes"]
+        # here
         run_docker(wd, project, cmd)
+        try:
+            t0 = time.time()
 
-        t0 = time.time()
-
-        client = check_docker_environment()
-        service2id = get_services_id(wd, project, services)
-        logger.debug(service2id=service2id)
-        for service_name, container_id in service2id.items():
-            container: Container = client.containers.get(container_id)
-            container.reload()
-            #
-            # cpu.cfs_quota_us: the total available run-time within a period (in microseconds)
-            # cpu.cfs_period_us: the length of a period (in microseconds)
-            # cpu.stat: exports throttling statistics [explained further below]
-
-            container.update(cpu_shares=250)
-
-            if is_solution_container(service_name):
-                M = "8g"
-                container.update(memswap_limit=-1, mem_reservation=M, mem_limit=M)
-                if quota_cpu is not None:
-                    # every second
-                    cpu_period = 1000 * 1000
-                    # get 300% cpu
-                    cpu_quota = int(quota_cpu * 1000 * 1000)
-
-                    container.update(cpu_period=cpu_period, cpu_quota=cpu_quota)
-                    msg = "Applying CPU quota"
-                    logger.info(
-                        msg, service_name=service_name, cpu_period=cpu_period, cpu_quota=cpu_quota,
-                    )
-
-        logger.debug(f"Starting services", services=services)
-        cmd = ["up", "-d"]
-        run_docker(wd, project, cmd)
-
-        visualization_info: Dict[ServiceName, str] = {}
-        for service_name, container_id in service2id.items():
-            container: Container = client.containers.get(container_id)
-            container.reload()
-            # noinspection PyUnresolvedReferences
-            ports = container.ports
-            # {'10123/tcp': [{'HostIp': '0.0.0.0', 'HostPort': '32769'}]}
-            logger.debug(f"container ports", service_name=service_name, ports=container.ports)
-            for port_proto, forwards in ports.items():
-                for forward in forwards:
-                    if "HostPort" in forward:
-                        HostPort = forward["HostPort"]
-
-                        msg = f"   Visualization will be available shortly at http://127.0.0.1:{HostPort}   "
-                        # msg = termcolor.colored(msg, color="blue")
-                        msg = "\n" * 10 + box(msg) + "\n" * 2
-                        logger.info(msg)
-                        visualization_info[service_name] = f"http://127.0.0.1:{HostPort}"
-
-        l = max(len(_) for _ in service2id)
-
-        colors_system = ["purple", "blue", "yellow", "magenta"]
-        colors_solutions = ["green"]
-
-        for i, (service_name, service_id) in enumerate(service2id.items()):
-            if quiet and (not is_solution_container(service_name)):
-                continue
-            if is_solution_container(service_name):
-                series = colors_solutions
-            else:
-                series = colors_system
-            color = series[i % len(series)]
-            stream_logs(wd, service_name, service_id, color=color, l=l)
-        exited2 = {}
-        heartbeat_interval = 10.0
-        last_heartbeat = 0.0
-
-        extra = random.uniform(0, 1)
-        while True:
-            time.sleep(5 + extra + random.uniform(0, 1))
-
-            if heartbeat:
-                t = time.time()
-                if t - last_heartbeat > heartbeat_interval:
-                    heartbeat()
-                last_heartbeat = t
-            running = []
-            running_details = {}
+            client = check_docker_environment()
+            service2id = get_services_id(wd, project, services)
+            logger.debug(service2id=service2id)
             for service_name, container_id in service2id.items():
-                if service_name in exited2:
-                    continue
-                container = client.containers.get(container_id)
+                container: Container = client.containers.get(container_id)
+                container.reload()
+                #
+                # cpu.cfs_quota_us: the total available run-time within a period (in microseconds)
+                # cpu.cfs_period_us: the length of a period (in microseconds)
+                # cpu.stat: exports throttling statistics [explained further below]
 
-                stats = container.stats(stream=False)
-                try:
-                    memory_stats1 = int(stats["memory_stats"]["usage"] / (1024 * 1024.0))
-                    memory_stats = f"{memory_stats1} MB RAM"
-                except KeyError:
-                    memory_stats = "n/a"
-                cpu_stats = stats["cpu_stats"]["cpu_usage"]["total_usage"] / (1024 * 1024.0)
-                running_details[service_name] = f"{memory_stats}; cpu {int(cpu_stats)}"
+                container.update(cpu_shares=250)
+
+                if is_solution_container(service_name):
+                    M = "8g"
+                    container.update(memswap_limit=-1, mem_reservation=M, mem_limit=M)
+                    if quota_cpu is not None:
+                        # every second
+                        cpu_period = 1000 * 1000
+                        # get 300% cpu
+                        cpu_quota = int(quota_cpu * 1000 * 1000)
+
+                        container.update(cpu_period=cpu_period, cpu_quota=cpu_quota)
+                        msg = "Applying CPU quota"
+                        logger.info(
+                            msg, service_name=service_name, cpu_period=cpu_period, cpu_quota=cpu_quota,
+                        )
+
+            logger.debug(f"Starting services", services=services)
+            cmd = ["up", "-d"]
+            run_docker(wd, project, cmd)
+
+            visualization_info: Dict[ServiceName, str] = {}
+            for service_name, container_id in service2id.items():
+                container: Container = client.containers.get(container_id)
+                container.reload()
+                # noinspection PyUnresolvedReferences
+                ports = container.ports
+                # {'10123/tcp': [{'HostIp': '0.0.0.0', 'HostPort': '32769'}]}
+                logger.debug(f"container ports", service_name=service_name, ports=container.ports)
+                for port_proto, forwards in ports.items():
+                    for forward in forwards:
+                        if "HostPort" in forward:
+                            HostPort = forward["HostPort"]
 
-                if container.status == "running":
-                    running.append(service_name)
-                else:
-                    container_state = container.attrs["State"]
-                    exit_code = container_state["ExitCode"]
-                    finished_at = parse(container_state["FinishedAt"])
-                    exited2[service_name] = exit_code
-                    if exit_code:
-                        # wait a little bit before bailing out
-                        # maybe the evaluator has better debugging
-                        passed = now_utc() - finished_at
-                        if (exit_code not in [0, 137, 138, 139]) and passed.total_seconds() < 15:
                             msg = (
-                                f"Container for service {service_name} has exited with code {exit_code}. "
-                                "Waiting a bit before bailing out."
+                                f"   Visualization will be available shortly at http://127.0.0.1:"
+                                f"{HostPort}   "
                             )
+                            # msg = termcolor.colored(msg, color="blue")
+                            msg = "\n" * 10 + box(msg) + "\n" * 2
                             logger.info(msg)
-                        else:
-                            msg = f'The container "{service_name}" exited with code {exit_code}.\n'
-                            if exit_code == 137:
-                                msg += (
-                                    "\n\nError code 137 means an out-of-memory error. "
-                                    "The container exceeded the memory available to it. "
+                            visualization_info[service_name] = f"http://127.0.0.1:{HostPort}"
+
+            l = max(len(_) for _ in service2id)
+
+            colors_system = ["purple", "blue", "yellow", "magenta"]
+            colors_solutions = ["green"]
+
+            for i, (service_name, service_id) in enumerate(service2id.items()):
+                if quiet and (not is_solution_container(service_name)):
+                    continue
+                if is_solution_container(service_name):
+                    series = colors_solutions
+                else:
+                    series = colors_system
+                color = series[i % len(series)]
+                stream_logs(wd, service_name, service_id, color=color, l=l)
+            exited2 = {}
+            heartbeat_interval = 10.0
+            last_heartbeat = 0.0
+
+            extra = random.uniform(0, 1)
+            while True:
+                time.sleep(5 + extra + random.uniform(0, 1))
+
+                if heartbeat:
+                    t = time.time()
+                    if t - last_heartbeat > heartbeat_interval:
+                        heartbeat()
+                    last_heartbeat = t
+                running = []
+                running_details = {}
+                for service_name, container_id in service2id.items():
+                    if service_name in exited2:
+                        continue
+                    container = client.containers.get(container_id)
+
+                    stats = container.stats(stream=False)
+                    try:
+                        memory_stats1 = int(stats["memory_stats"]["usage"] / (1024 * 1024.0))
+                        memory_stats = f"{memory_stats1} MB RAM"
+                    except KeyError:
+                        memory_stats = "n/a"
+                    cpu_stats = stats["cpu_stats"]["cpu_usage"]["total_usage"] / (1024 * 1024.0)
+                    running_details[service_name] = f"{memory_stats}; cpu {int(cpu_stats)}"
+
+                    if container.status == "running":
+                        running.append(service_name)
+                    else:
+                        container_state = container.attrs["State"]
+                        exit_code = container_state["ExitCode"]
+                        finished_at = parse(container_state["FinishedAt"])
+                        exited2[service_name] = exit_code
+                        if exit_code:
+                            # wait a little bit before bailing out
+                            # maybe the evaluator has better debugging
+                            passed = now_utc() - finished_at
+                            if (exit_code not in [0, 137, 138, 139]) and passed.total_seconds() < 15:
+                                msg = (
+                                    f"Container for service {service_name} has exited with code {exit_code}. "
+                                    "Waiting a bit before bailing out."
                                 )
-                                status = ChallengesConstants.STATUS_JOB_HOST_ERROR
-                                cr = ChallengeResults(status, msg, scores={})
-                                return cr
-                            if exit_code == 138:
-                                msg += "\n\nError code 138 means that GPU could not be found."
-                                status = ChallengesConstants.STATUS_JOB_HOST_ERROR
-                                cr = ChallengeResults(status, msg, scores={})
-                                return cr
-                            if exit_code == 139:
-                                msg += "\n\nError code 139 means GPU memory error."
+                                logger.info(msg)
+                            else:
+                                msg = f'The container "{service_name}" exited with code {exit_code}.\n'
+                                if exit_code == 137:
+                                    msg += (
+                                        "\n\nError code 137 means an out-of-memory error. "
+                                        "The container exceeded the memory available to it. "
+                                    )
+                                    status = ChallengesConstants.STATUS_JOB_HOST_ERROR
+                                    cr = ChallengeResults(status, msg, scores={})
+                                    return cr
+                                if exit_code == 138:
+                                    msg += "\n\nError code 138 means that GPU could not be found."
+                                    status = ChallengesConstants.STATUS_JOB_HOST_ERROR
+                                    cr = ChallengeResults(status, msg, scores={})
+                                    return cr
+                                if exit_code == 139:
+                                    msg += "\n\nError code 139 means GPU memory error."
+                                    status = ChallengesConstants.STATUS_JOB_HOST_ERROR
+                                    cr = ChallengeResults(status, msg, scores={})
+                                    return cr
+
+                                msg += "\n\nLook at the logs for the container to know more about the error."
+                                logger.error(msg)
+
+                                if is_solution_container(service_name):
+                                    status = ChallengesConstants.STATUS_JOB_FAILED
+                                    cr = ChallengeResults(status, msg, scores={})
+                                    return cr
+
                                 status = ChallengesConstants.STATUS_JOB_HOST_ERROR
                                 cr = ChallengeResults(status, msg, scores={})
                                 return cr
 
-                            msg += "\n\nLook at the logs for the container to know more about the error."
-                            logger.error(msg)
+                delta = time.time() - t0
+                msg = f"Running for {int(delta)} s (timeout: {int(timeout_sec)} s)."
+                logger.info(
+                    msg,
+                    running_details=running_details,
+                    exited=exited2,
+                    visualization_info=visualization_info,
+                )
 
-                            if is_solution_container(service_name):
-                                status = ChallengesConstants.STATUS_JOB_FAILED
-                                cr = ChallengeResults(status, msg, scores={})
-                                return cr
+                if monitor in exited2:
+                    logger.info(f"The container to monitor {monitor!r} has exited; bye bye.", exited=exited2)
+                    break
 
-                            status = ChallengesConstants.STATUS_JOB_HOST_ERROR
-                            cr = ChallengeResults(status, msg, scores={})
-                            return cr
+                if delta > timeout_sec:
+                    msg = f"Waited {int(delta)} seconds for container to finish. Giving up. "
+                    logger.error(msg)
+                    status = ChallengesConstants.STATUS_JOB_ERROR
+                    cr = ChallengeResults(status, msg, scores={})
+                    return cr
 
-            delta = time.time() - t0
-            msg = f"Running for {int(delta)} s (timeout: {int(timeout_sec)} s)."
-            logger.info(
-                msg, running_details=running_details, exited=exited2, visualization_info=visualization_info
-            )
+            try:
+                cr = read_challenge_results(wd)
+            except NoResultsFound as e:
+                msg = f"""\
+    The result file is not found in working dir {wd}:
 
-            if monitor in exited2:
-                logger.info(f"The container to monitor {monitor!r} has exited; bye bye.", exited=exited2)
-                break
+    {e}
 
-            if delta > timeout_sec:
-                msg = f"Waited {int(delta)} seconds for container to finish. Giving up. "
+    This usually means that the evaluator did not finish and some times that there was an import error.
+    Check the evaluator log to see what happened."""
+                l = "\n".join("-" + f for f in list_all_files(wd))
+                msg += f"\n\nList of all files:\n\n {l}"
                 logger.error(msg)
                 status = ChallengesConstants.STATUS_JOB_ERROR
                 cr = ChallengeResults(status, msg, scores={})
-                return cr
 
-        try:
-            cr = read_challenge_results(wd)
-        except NoResultsFound as e:
-            msg = f"""\
-The result file is not found in working dir {wd}:
-
-{e}
-
-This usually means that the evaluator did not finish and some times that there was an import error.
-Check the evaluator log to see what happened."""
-            l = "\n".join("-" + f for f in list_all_files(wd))
-            msg += f"\n\nList of all files:\n\n {l}"
-            logger.error(msg)
-            status = ChallengesConstants.STATUS_JOB_ERROR
-            cr = ChallengeResults(status, msg, scores={})
+        finally:
+            pass
+            # logger.info("calling down")
+            # try:
+            #
+            #     cmd = ["down", "-v"]
+            #     run_docker(wd, project, cmd)
+            # except:
+            #     logger.warn(traceback.format_exc())
 
     except DockerComposeFail as e:
         msg = f"Error while running Docker Compose:\n\n{e}"
